unit DosAdventure.Embedded;

interface

uses
  DosAdventure;

type
  TDosAdventureEmbeddedFactory = class(TDosAdventureFactory)
  private
    FUseSync: Boolean;
  protected
    function InternalCreateDosAdventure: TDosAdventure; override;
    property UseSync: Boolean read FUseSync;
  public
    constructor Create(AUseSync: Boolean);
  end;

implementation

uses
  System.SysUtils, System.StrUtils, System.Classes, DosCommand;

type
  TDosAdventureEmbedded = class(TDosAdventure)
  private
    FDosCommand: TDosCommand;
    FSkipNext: string;
    FBuffer: string;
    procedure DoNewLine(ASender: TObject; const ANewLine: string; AOutputType: TOutputType);
    procedure DoTerminated(Sender: TObject);
    procedure HandleResponse;
    procedure AddLine(const ANewLine: string);
    procedure CheckPartialLine(const ANewLine: string);
  protected
    procedure InternalNotifyResponse(const AResponse: string); virtual;
  public
    constructor Create;
    destructor Destroy; override;
    function IsRunning: Boolean; override;
    procedure SendText(const AText: string); override;
    procedure Start; override;
  end;

  TDosAdventureEmbeddedSync = class(TDosAdventureEmbedded)
  protected
    procedure InternalNotifyResponse(const AResponse: string); override;
  end;

constructor TDosAdventureEmbedded.Create;
begin
  inherited;
  FDosCommand := TDosCommand.Create(nil);
  FDosCommand.CurrentDir := '';
  FDosCommand.OnNewLine := DoNewLine;
  FDosCommand.OnTerminated := DoTerminated;
end;

destructor TDosAdventureEmbedded.Destroy;
begin
  FDosCommand.Free;
  inherited Destroy;
end;

procedure TDosAdventureEmbedded.AddLine(const ANewLine: string);
var
  startIndex: Integer;
begin
  startIndex := 0;
  { FSkipNext haben wir bereits ausgegeben. }
  if (FSkipNext > '') and ANewLine.StartsWith(FSkipNext) then begin
    startIndex := Length(FSkipNext);
  end;
  FSkipNext := '';
  FBuffer := FBuffer + ANewLine.Substring(startIndex) + sLineBreak;
end;

procedure TDosAdventureEmbedded.CheckPartialLine(const ANewLine: string);
begin
  if MatchStr(ANewLine, ['* ', '** ']) then begin
    { Jetzt wartet Adventure auf eine Eingabe.
      Eine Funktion in TDosCommand wie IsWaitingForInput wre mir lieber. }
    FBuffer := FBuffer + ANewLine;
    { Was wir jetzt anfgen, mssen wir bei der nchsten ganzen Zeile berspringen. }
    FSkipNext := ANewLine;
    HandleResponse;
  end;
end;

procedure TDosAdventureEmbedded.HandleResponse;
var
  response: string;
begin
  response := FBuffer;
  FBuffer := '';
  InternalNotifyResponse(response);
end;

procedure TDosAdventureEmbedded.DoNewLine(ASender: TObject; const ANewLine: string; AOutputType: TOutputType);
begin
  case AOutputType of
    otEntireLine:
      AddLine(ANewLine);
    otBeginningOfLine:
      CheckPartialLine(ANewLine);
  end;
end;

procedure TDosAdventureEmbedded.DoTerminated(Sender: TObject);
begin
  HandleResponse;
end;

function TDosAdventureEmbedded.IsRunning: Boolean;
begin
  Result := (FDosCommand <> nil) and FDosCommand.IsRunning;
end;

procedure TDosAdventureEmbedded.SendText(const AText: string);
begin
  FBuffer := '';
  FDosCommand.SendLine(AText, true);
end;

procedure TDosAdventureEmbedded.Start;
begin
  FBuffer := '';
  FDosCommand.CommandLine := 'advent';
  { Wenn das Execute nicht aus dem Hauptthread aufgerufen wird, kommt es
    beim OnTerminate zu einer Exception! }
  TThread.Synchronize(nil,
    procedure
    begin
      FDosCommand.Execute;
    end);
end;

procedure TDosAdventureEmbedded.InternalNotifyResponse(const AResponse: string);
begin
  NotifyResponse(AResponse);
end;

procedure TDosAdventureEmbeddedSync.InternalNotifyResponse(const AResponse: string);
begin
  TThread.Queue(nil,
    procedure
    begin
      NotifyResponse(AResponse);
    end);
end;

constructor TDosAdventureEmbeddedFactory.Create(AUseSync: Boolean);
begin
  inherited Create;
  FUseSync := AUseSync;
end;

function TDosAdventureEmbeddedFactory.InternalCreateDosAdventure: TDosAdventure;
begin
  if UseSync then begin
    Result := TDosAdventureEmbeddedSync.Create;
  end
  else begin
    Result := TDosAdventureEmbedded.Create;
  end;
end;

end.
